@@ -39,7 +39,7 @@ public:
   //! \brief get type
   //! \return type as string
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  const std::string
+  std::string
   get_type () const override
   {
     return "gpt";
