@@ -20,9 +20,9 @@
 //! \file data_decoder.cc C++ API <i>mobius.decoder.data_decoder</i> class wrapper
 //! \author Eduardo Aguiar
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+#include <pymobius.h>
 #include "data_decoder.h"
 #include "module.h"
-#include "api.h"
 #include "io/reader.h"
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -171,7 +171,7 @@ tp_f_get_int8 (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyInt_FromLong (self->obj->get_int8 ());
+      ret = PyLong_FromLong (self->obj->get_int8 ());
     }
   catch (const std::exception& e)
     {
@@ -196,7 +196,7 @@ tp_f_get_int16_le (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyInt_FromLong (self->obj->get_int16_le ());
+      ret = PyLong_FromLong (self->obj->get_int16_le ());
     }
   catch (const std::exception& e)
     {
@@ -246,7 +246,7 @@ tp_f_get_uint8 (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyInt_FromLong (self->obj->get_uint8 ());
+      ret = PyLong_FromLong (self->obj->get_uint8 ());
     }
   catch (const std::exception& e)
     {
@@ -271,7 +271,7 @@ tp_f_get_uint16_le (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyInt_FromLong (self->obj->get_uint16_le ());
+      ret = PyLong_FromLong (self->obj->get_uint16_le ());
     }
   catch (const std::exception& e)
     {
@@ -296,7 +296,7 @@ tp_f_get_uint32_le (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyInt_FromLong (self->obj->get_uint32_le ());
+      ret = PyLong_FromLong (self->obj->get_uint32_le ());
     }
   catch (const std::exception& e)
     {
@@ -346,7 +346,7 @@ tp_f_get_uint16_be (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyInt_FromLong (self->obj->get_uint16_be ());
+      ret = PyLong_FromLong (self->obj->get_uint16_be ());
     }
   catch (const std::exception& e)
     {
@@ -371,7 +371,7 @@ tp_f_get_uint32_be (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyInt_FromLong (self->obj->get_uint32_be ());
+      ret = PyLong_FromLong (self->obj->get_uint32_be ());
     }
   catch (const std::exception& e)
     {
@@ -584,7 +584,7 @@ tp_f_get_string_by_size (decoder_data_decoder_o *self, PyObject *args)
 
   try
     {
-      ret = PyString_FromString (self->obj->get_string_by_size (arg_size, arg_encode).c_str ());
+      ret = PyString_from_std_string (self->obj->get_string_by_size (arg_size, arg_encode));
     }
   catch (const std::exception& e)
     {
@@ -615,7 +615,7 @@ tp_f_get_c_string (decoder_data_decoder_o *self, PyObject *args)
 
   try
     {
-      ret = PyString_FromString (self->obj->get_c_string (arg_encode).c_str ());
+      ret = PyString_from_std_string (self->obj->get_c_string (arg_encode));
     }
   catch (const std::exception& e)
     {
@@ -646,7 +646,7 @@ tp_f_get_hex_string_by_size (decoder_data_decoder_o *self, PyObject *args)
 
   try
     {
-      ret = PyString_FromString (self->obj->get_hex_string_by_size (arg_size).c_str ());
+      ret = PyString_from_std_string (self->obj->get_hex_string_by_size (arg_size));
     }
   catch (const std::exception& e)
     {
@@ -671,7 +671,7 @@ tp_f_get_guid (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyString_FromString (self->obj->get_guid ().c_str ());
+      ret = PyString_from_std_string (self->obj->get_guid ());
     }
   catch (const std::exception& e)
     {
@@ -696,7 +696,32 @@ tp_f_get_uuid (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyString_FromString (self->obj->get_uuid ().c_str ());
+      ret = PyString_from_std_string (self->obj->get_uuid ());
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+    }
+
+  // return value
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief <i>get_sid</i> method implementation
+//! \param self object
+//! \param args argument list
+//! \return SID as string
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_get_sid (decoder_data_decoder_o *self, PyObject *)
+{
+  // execute C++ function
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = PyString_from_std_string (self->obj->get_sid ());
     }
   catch (const std::exception& e)
     {
@@ -721,7 +746,7 @@ tp_f_get_ipv4 (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyString_FromString (self->obj->get_ipv4 ().c_str ());
+      ret = PyString_from_std_string (self->obj->get_ipv4 ());
     }
   catch (const std::exception& e)
     {
@@ -746,7 +771,7 @@ tp_f_get_ipv4_be (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyString_FromString (self->obj->get_ipv4_be ().c_str ());
+      ret = PyString_from_std_string (self->obj->get_ipv4_be ());
     }
   catch (const std::exception& e)
     {
@@ -771,7 +796,7 @@ tp_f_get_ipv4_le (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyString_FromString (self->obj->get_ipv4_le ().c_str ());
+      ret = PyString_from_std_string (self->obj->get_ipv4_le ());
     }
   catch (const std::exception& e)
     {
@@ -937,6 +962,12 @@ static PyMethodDef tp_methods[] =
     METH_VARARGS,
     "get UUID"
   },
+  {
+    (char *) "get_sid",
+    (PyCFunction) tp_f_get_sid,
+    METH_VARARGS,
+    "Get SID"
+  },
   {
     (char *) "get_ipv4",
     (PyCFunction) tp_f_get_ipv4,
@@ -970,15 +1001,32 @@ tp_new (PyTypeObject *type, PyObject *args, PyObject *)
 {
   // parse input args
   PyObject *arg_reader = nullptr;
+  const std::uint8_t *arg_data_buffer = nullptr;
+  Py_ssize_t arg_data_size = 0;
 
-  if (!PyArg_ParseTuple (args, "O!", &io_reader_t, &arg_reader))
+  if (!PyArg_ParseTuple (args, "s#", &arg_data_buffer, &arg_data_size) &&
+      !PyArg_ParseTuple (args, "O!", &io_reader_t, &arg_reader))
     return nullptr;
 
+  PyErr_Clear ();
+
   // create object
   decoder_data_decoder_o *self = (decoder_data_decoder_o *) type->tp_alloc (type, 0);
 
   if (self)
-    self->obj = new mobius::decoder::data_decoder (*((io_reader_o *) arg_reader)->obj);
+    {
+      if (arg_data_buffer)
+        {
+          self->data = new mobius::bytearray (arg_data_buffer, arg_data_size);
+          self->obj = new mobius::decoder::data_decoder (* (self->data));
+        }
+         
+      else
+        {
+          self->obj = new mobius::decoder::data_decoder (PyMobius_IO_Reader_as_cpp (arg_reader));
+          self->data = nullptr;
+        }
+    }
 
   return (PyObject *) self;
 }
@@ -991,6 +1039,7 @@ static void
 tp_dealloc (decoder_data_decoder_o *self)
 {
   delete self->obj;
+  delete self->data;
   self->ob_type->tp_free ((PyObject*) self);
 }
 
