@@ -20,10 +20,29 @@
 //! \brief C++ API module wrapper
 //! \author Eduardo Aguiar
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+#include <pymobius.h>
 #include "folder.h"
 #include "resource.h"
 #include "file.h"
-#include "api.h"
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Create file or folder object according to resource type
+//! \param obj Resource object
+//! \return Python object
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+PyMobius_IO_File_or_Folder_from_cpp (mobius::io::resource obj)
+{
+  PyObject *ret = nullptr;
+
+  if (obj.is_file ())
+    ret = PyMobius_IO_File_from_cpp (mobius::io::file (obj));
+
+  else
+    ret = PyMobius_IO_Folder_from_cpp (mobius::io::folder (obj));
+
+  return ret;
+}
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief create new object from C++ object
@@ -57,38 +76,18 @@ PyMobius_IO_Folder_from_cpp (mobius::io::folder obj)
 static PyObject *
 tp_getter_children (io_folder_o *self)
 {
-  PyObject *ret = PyList_New (0);
-
-  if (ret == nullptr)
-    return nullptr;
+  PyObject *ret = nullptr;
 
   try
     {
-      for (const auto& child : *(self->obj))
-        {
-          PyObject *item = nullptr;
-
-          if (child.is_file ())
-            item = PyMobius_IO_File_from_cpp (mobius::io::file (child));
-
-          else
-            item = PyMobius_IO_Folder_from_cpp (mobius::io::folder (child));
-
-          if (item == nullptr)
-            {
-              Py_CLEAR (ret);
-              return nullptr;
-            }
-
-          PyList_Append (ret, item);
-          Py_DECREF (item);
-        }
+      ret = PyList_from_cpp_container (
+               *(self->obj),
+               PyMobius_IO_File_or_Folder_from_cpp
+            );
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_IOError, e.what ());
-      Py_CLEAR (ret);
-      ret = nullptr;
+      PyErr_SetString (PyExc_Exception, e.what ());
     }
 
   return ret;
